@@ -1,6 +1,6 @@
 import sys
 import os
-import urllib
+import urllib2
 import urlparse
 import xml.sax.saxutils
 from paste.script import templates
@@ -8,15 +8,17 @@ from paste.script.templates import NoDefault
 from grokproject.utils import run_buildout
 from grokproject.utils import ask_var
 from grokproject.utils import get_boolean_value_for_option
+from grokproject.utils import get_sha1_encoded_string
 from grokproject.utils import create_buildout_default_file
 from grokproject.utils import exist_buildout_default_file
 from grokproject.utils import required_grok_version
+from grokproject.utils import extend_versions_cfg
 
-GROK_RELEASE_URL = 'http://grok.zope.org/releaseinfo/'
+GROK_RELEASE_URL_DEFAULT = 'http://grok.zope.org/releaseinfo/'
 
 
 class GrokProject(templates.Template):
-    _template_dir = 'template'
+    _template_dir = 'template_paste'
     summary = "A grok project"
     required_templates = []
 
@@ -35,6 +37,10 @@ class GrokProject(templates.Template):
         ask_var('eggs_dir',
                 'Location where zc.buildout will look for and place packages',
                 default='', should_ask=False),
+        ask_var('grok_release_url',
+                "URL where grokproject will look up grok version and "
+                "release information.",
+                should_ask=False),
         ]
 
     def check_vars(self, vars, cmd):
@@ -46,6 +52,10 @@ class GrokProject(templates.Template):
             sys.exit(1)
 
         explicit_eggs_dir = vars.get('eggs_dir')
+        grok_release_url = vars.get('grok_release_url',
+                                    GROK_RELEASE_URL_DEFAULT)
+        if not grok_release_url.endswith('/'):
+            grok_release_url += '/'
 
         skipped_vars = {}
         for var in list(self.vars):
@@ -57,29 +67,37 @@ class GrokProject(templates.Template):
         for name in skipped_vars:
             vars[name] = skipped_vars[name]
 
+        vars['grok_release_url'] = grok_release_url
+
+        vars['passwd'] = get_sha1_encoded_string(vars['passwd'])            
         for var_name in ['user', 'passwd']:
             # Escape values that go in site.zcml.
             vars[var_name] = xml.sax.saxutils.quoteattr(vars[var_name])
         vars['app_class_name'] = vars['project'].capitalize()
 
+        create_zopectl = False
+        if vars.get('zopectl','') == 'True':
+            create_zopectl = True
+        if create_zopectl:
+            self._template_dir = 'template_zopectl'
+
         # Handling the version.cfg file.
         print "Downloading info about versions..."
-        current_info_url = GROK_RELEASE_URL + 'current'
-        try:
-            info = urllib.urlopen(current_info_url).read().strip()
-        except IOError:
-            print "Error: cannot download required %s" % current_info_url
-            print "Server may be down.  Please try again later."
-            sys.exit(1)
-        version_info_url = urlparse.urljoin(current_info_url, info)
+        version = vars.get('grokversion', 'current')
+        cfg_filename = 'grok-%s.cfg' % version
+        if version == 'current':
+            # if no version was specified, we look up the current
+            # version first
+            current_info_url = urlparse.urljoin(grok_release_url, 'current')
+            cfg_filename = self.download(current_info_url).strip()
+
+        version_info_url = urlparse.urljoin(grok_release_url, cfg_filename)
         vars['version_info_url'] = version_info_url
-        try:
-            version_info_file_contents = urllib.urlopen(version_info_url).read()
-        except IOError:
-            print "Error: cannot download required %s" % version_info_url
-            print "Server may be down.  Please try again later."
-            sys.exit(1)
-        vars['version_info_file_contents'] = version_info_file_contents
+        vars['version_info_file_contents'] = self.download(version_info_url)
+
+        # Maybe add additional eggs...
+        vars['version_info_additions'] = extend_versions_cfg(
+            vars['version_info_file_contents'], create_zopectl)
 
         # Which grok version are we depending on?
         version = required_grok_version(vars['version_info_file_contents'])
@@ -98,10 +116,36 @@ class GrokProject(templates.Template):
         else:
             create_buildout_default_file()
 
+        vars['package_directory'] = os.path.abspath(os.path.join(
+                os.getcwd(), vars['project']))
+        
         return vars
+    
+    def download(self, url):
+        """Downloads a file and returns the contents.
 
+        If an error occurs, we abort, giving some information about
+        the reason.
+        """
+        contents = None
+        try:
+            contents = urllib2.urlopen(url).read()
+        except urllib2.HTTPError:
+            # Some 404 or similar happened...
+            print "Error: cannot download required %s" % url
+            print "Maybe you specified a non-existing version?"
+            sys.exit(1)
+        except IOError, e:
+            # Some serious problem: no connect to server...
+            print "Error: cannot download required %s" % version_info_url
+            print "Server may be down.  Please try again later."
+            sys.exit(1)
+        return contents
+    
     def post(self, command, output_dir, vars):
         if not vars['run_buildout']:
             return
+        original_dir = os.getcwd()
         os.chdir(vars['project'])
         run_buildout(command.options.verbose)
+        os.chdir(original_dir)
